Module: Finrb
- Includes:
- Newton
- Defined in:
- lib/finrb.rb,
lib/finrb/rates.rb,
lib/finrb/utils.rb,
lib/finrb/config.rb,
lib/finrb/cashflows.rb,
lib/finrb/transaction.rb,
lib/finrb/amortization.rb
Overview
The Finrb module adheres to the following conventions for financial calculations:
- Positive values represent cash inflows (money received); negative values represent cash outflows (payments).
- principal represents the outstanding balance of a loan or annuity.
- rate represents the interest rate per period.
Defined Under Namespace
Modules: Cashflow Classes: Amortization, Interest, Payment, Rate, Transaction, Utils
Class Method Summary collapse
Class Method Details
.config ⇒ Object
7 8 9 |
# File 'lib/finrb/config.rb', line 7 def self.config @config ||= Config.new(eps: '1.0e-16', guess: 1.0, business_days: false, periodic_compound: false) end |
.configure {|config| ... } ⇒ Object
11 12 13 |
# File 'lib/finrb/config.rb', line 11 def self.configure yield(config) end |